﻿ // 
 // fillMetaDataFromStyles.js
 //
 // auteur : Emmanuel Côtez, Publications scientifiques du Muséum national d'Histoire naturelle, © 2019
 // 
 //
 // Ce script renseigne les métadonnées d'un document InDesign à partir des styles de la première page
 // Il récupère les informations suivantes...
 	// – première page/Titre article ;
 	// – auteur(s) ;
 	// – numéro de volume ;
 	// – numéro d'article.
 // ...et renseigne les métadonnées du fichier InDesign (et donc du fichier PDF)
 //
 // --------------------
 // v. 1.01
 // Nouveautés :
 // – prise en charge des revues de Cryptogamie ;
 // – mise à jour du copyright 2019 ;
 // – génération automatique de l'hyperlien vers l'article. ex. : http://geodiversitas.com/41/6
 // --------------------
 // v. 1.25 : mise à jour 2022, correction du titre (prise en compte des sauts de ligne et des tirets)
 // --------------------
 
 // Fonctions
 	function trim (str) {
		return str.replace(/^\s+/,'').replace(/\s+$/,''); 
		} 
		
	function get_date_string(para_date) { // Fonction de récupération des date et heure actuelles
		var date = para_date || new Date();
		var day = (date.getDate() > 10 ) ? date.getDate() : ("0" + date.getDate() ),
			month = ((date.getMonth()+1) > 9 ) ? (date.getMonth()+1) : ("0" + (date.getMonth()+1) ),
			year = (date.getFullYear() > 9 ) ? date.getFullYear() : ("0" + date.getFullYear() ),
			hours = (date.getHours() > 9 ) ? date.getHours() : ("0" + date.getHours() ),
			minutes = (date.getMinutes() > 9 ) ? date.getMinutes() : ("0" + date.getMinutes() );
			return year + "-" + month +"-" + day + "T" + hours + ":" + minutes + ":" + "00" ;
			};
			
// fin des fonctions

var myDocument = app.activeDocument;
var nomRevue = new String ;

// Choix de la revue +-+-+-+-+-+-+-+-+-
var w, layout;    
layout = "dialog { dropdown: DropDownList { size: [120,20] properties: { items: ['Geodiversitas','Zoosystema','Adansonia, sér. 3,','Palevol','Anthropozoologica','Naturae', 'Cryptogamie, Algologie', 'Cryptogamie, Bryologie', 'Cryptogamie, Mycologie'] } } }";  
w = new Window(layout);  

w.dropdown.selection=0;  
w.dropdown.minimumSize.width = 200;
w.dropdown.minimumSize.height = 200;
w.dropdown.title = "Revue :";

w.add ("button", undefined, "OK"); 
w.show();         

nomRevue = w.dropdown.selection.text ;
nomRevueWeb = w.dropdown.selection.text.toLowerCase(); 

// Fin du choix de la revue +-+-+-+-+-+-+-+-+-

// --------------------------------------
// Récupération des données
// Variables renseignées :
// – title ;
// – auteurs (tableau de données) ;
// – resume (contient les résumés anglais et français à la suite);
// – motscles ;
// – keywords ;
// – volume ;
// – numarticle
// --------------------------------------
     
    // title +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    app.findGrepPreferences=app.changeGrepPreferences=null;
	// app.findGrepPreferences.findWhat="[\\w,&\\.\\s()]+"; // original sauvegardé 
	// app.findGrepPreferences.findWhat="[\\w,;\"'‘’«»”“&\\.\\:\\s()—–-]+";
	app.findGrepPreferences.findWhat="[\"'‘’«»”“\\w|\\s.,;:<>=\"\'?—&\\(\\)\\[\\]/!-]+";
    app.findGrepPreferences.appliedParagraphStyle = "Titre article";
    titleFromStyle = app.activeDocument.findGrep() ;
    title = titleFromStyle[0].contents;
    //
    title = title.replace(/\n/g," ");
	title = title.replace(/\r/g," ");
    title = title.replace(/\s+/g," ");
    // fin title +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-


    // auteurs (tableau de données) +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    app.findGrepPreferences=app.changeGrepPreferences=null;
    app.findGrepPreferences.findWhat="(.+)";
    app.findGrepPreferences.appliedParagraphStyle = "Auteur";
    var auteursFromStyle = new Array() ;     
    auteursFromStyle = app.activeDocument.findGrep() ;
    var lnAuteurs = auteursFromStyle.length ;
    var auteurs = "" ;
    
    // maximum : 25 auteurs pris en compte
    for (i=0 ; i<lnAuteurs ; i++) {
        if (i==0) { auteurs = auteurs + auteursFromStyle[i].contents ; }
        else {
            auteurs = auteurs + " ; " + auteursFromStyle[i].contents ;          
            }
        }
    // fin auteurs (tableau de données) +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
                
    // resume (Résumé et Abstract) +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    app.findGrepPreferences = app.changeGrepPreferences=null;
    app.findGrepPreferences.findWhat="(.+)";
    app.findGrepPreferences.appliedParagraphStyle = "Abstract/Texte";
    
    descriptionFromStyle = app.activeDocument.findGrep() ;
    // 
    resume = "English abstract (résumé anglais)\r" + descriptionFromStyle[0].contents ;

    app.findGrepPreferences = app.changeGrepPreferences = null;
    app.findGrepPreferences.findWhat="(.+)";
    app.findGrepPreferences.appliedParagraphStyle = "Résumé/Texte";
    descriptionFromStyle = app.activeDocument.findGrep() ;
    //
    resume = resume + "\r\rFrench abstract (résumé français)\r" + descriptionFromStyle[0].contents ;
    // fin resume (Résumé et Abstract) +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-


    // motscles & keywords +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    app.findGrepPreferences=app.changeGrepPreferences=null;
    app.findGrepPreferences.findWhat="(.+\\s?)";
    app.findGrepPreferences.appliedParagraphStyle = "Mots clés/Texte";
    
    motsclesFromStyle = app.activeDocument.findGrep() ;

    var motscles = new Array();

    for (i=0;i<motsclesFromStyle.length;i++) {
        motscles[i] = motsclesFromStyle[i].contents ;
        motscles[i] = motscles[i].replace(",","") ;
        motscles[i] = motscles[i].replace(".","") ;
        motscles[i] = motscles[i].replace("\r","") ;
        }
    // fin motscles & keywords +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-


    // volume +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    app.findGrepPreferences=app.changeGrepPreferences=null;
    app.findGrepPreferences.findWhat = "" + nomRevue + "\\s\\d+";
    app.findGrepPreferences.appliedParagraphStyle = "Citation";
    volumeFromStyle = app.activeDocument.findGrep() ;
    // alert(volumeFromStyle[0].contents);
    volume = volumeFromStyle[0].contents.split(nomRevue+" ") ;
    //
    volume = volume[1] ;
    // fin volume +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
	
    // année +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    app.findGrepPreferences=app.changeGrepPreferences=null;
    app.findGrepPreferences.findWhat = "(\\d{4})(\\s+)?$" ;
    app.findGrepPreferences.appliedParagraphStyle = "Date publication (1ère page)";
    anneeFromStyle = app.activeDocument.findGrep() ;
    
	try { annee = parseInt(trim(anneeFromStyle[0].contents.toString())); }
	catch (e) {
		var timestamp = get_date_string(new Date()) ;
		annee = timestamp.match(/\d{4}/);
		}
	
	annee = prompt("Confirmez ou corrigez l'année de publication :",annee) ;
	
	// alert (annee);
    // fin année +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-	

    // article +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
    app.findGrepPreferences=app.changeGrepPreferences=null;
    app.findGrepPreferences.findWhat = "" + nomRevue + "\\s\\d+\\s\\(\\d+\\)";

    app.findGrepPreferences.appliedParagraphStyle = "Citation";
    numarticleFromStyle = app.activeDocument.findGrep() ;

    numarticle = numarticleFromStyle[0].contents.split("(") ;
    numarticle = numarticle[1].split(")") ;
    //
    numarticle = numarticle[0] ; 
    // fin article +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
   

// -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+    
// Renseignement des métadonnées du fichier InDesign à partir des informations collectées
// -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

    with (myDocument.metadataPreferences){  
        author = auteurs ;
        if (nomRevueWeb == "geodiversitas") { copyrightInfoURL = "http://geodiversitas.com/"+volume+"/"+numarticle ; email="geodiv@mnhn.fr" ;   } 
        if (nomRevueWeb == "adansonia, sér. 3,") { 
        	copyrightInfoURL = "http://adansonia.com/"+volume+"/"+numarticle;  email="adanson@mnhn.fr" ; 
	        copyrightNotice = "© Publications scientifiques du Muséum national d’Histoire naturelle, Paris, "+annee+" — Adansonia — http://sciencepress.mnhn.fr" ; 
        	} 
		if (nomRevueWeb == "palevol") { 
        	copyrightInfoURL = "http://cr-palevol.fr/"+volume+"/"+numarticle;  email="emmanuel.cotez@mnhn.fr" ; 
	        copyrightNotice = "© Publications scientifiques du Muséum national d’Histoire naturelle, Académie des sciences, Paris, "+annee+" — Comptes Rendus Palevol — http://sciencepress.mnhn.fr" ; 
        	}         	
        if (nomRevueWeb == "zoosystema") { copyrightInfoURL = "http://zoosystema.com/"+volume+"/"+numarticle;  email="zoosyst@mnhn.fr" ; } 
        if (nomRevueWeb == "naturae") { copyrightInfoURL = "http://www.revue-naturae.fr/"+volume+"/"+numarticle;  email="naturae@mnhn.fr" ; } 
        if (nomRevueWeb == "anthropozoologica") { copyrightInfoURL = "http://anthropozoologica.com/"+volume+"/"+numarticle;  email="anthropo@mnhn.fr" ; } 
        if (nomRevueWeb == "cryptogamie, algologie") { copyrightInfoURL = "http://cryptogamie.com/algologie/"+volume+"/"+numarticle;  email="bryo@cryptogamie.com" ; } 
        if (nomRevueWeb == "cryptogamie, bryologie") { copyrightInfoURL = "http://cryptogamie.com/bryologie/"+volume+"/"+numarticle;  email="algo@cryptogamie.com" ; } 
        if (nomRevueWeb == "cryptogamie, mycologie") { copyrightInfoURL = "http://cryptogamie.com/mycologie/"+volume+"/"+numarticle;  email="myco@cryptogamie.com" ; } 
        
       if (nomRevueWeb != "adansonia, sér. 3," && nomRevueWeb != "palevol") { 
        copyrightNotice = "© Publications scientifiques du Muséum national d’Histoire naturelle, Paris, "+annee+" — " + nomRevue + " — http://sciencepress.mnhn.fr" ; 
    }
        copyrightStatus = CopyrightStatus.yes;  
        description = resume ;
        documentTitle = title ;  
        jobName = "Muséum national d'Histoire naturelle, Publications scientifiques, " + nomRevue;  
        keywords = motscles ;

        var myNewContainer = createContainerItem("http://ns.adobe.com/xap/1.0/", "email");  
        setProperty("http://ns.adobe.com/xap/1.0/", "email/*[1]",email);  
        } 
    
// -+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

// Message de confirmation final
alert("Les métadonnées du document sont renseignées\r(CMD+ALT+MAJ+i pour vérifier)\r\r[N'oubliez pas de SUPPRIMER LES GUILLEMETS autour des auteurs]");